var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { BigNumberString, ConsentFactoryContractError, EVMContractAddress, IpfsCID, MarketplaceListing, UnixTimestamp, } from "@snickerdoodlelabs/objects";
import { ethers } from "ethers";
import { injectable } from "inversify";
import { ResultAsync } from "neverthrow";
import { ResultUtils } from "neverthrow-result-utils";
import { BaseContract } from "../implementations/BaseContract.js";
import { ContractsAbis, } from "../interfaces/index.js";
let ConsentFactoryContract = class ConsentFactoryContract extends BaseContract {
    constructor(providerOrSigner, contractAddress) {
        super(providerOrSigner, contractAddress, ContractsAbis.ConsentFactoryAbi.abi);
        this.providerOrSigner = providerOrSigner;
        this.contractAddress = contractAddress;
    }
    getContractAddress() {
        return this.contractAddress;
    }
    createConsent(ownerAddress, baseUri, name, overrides) {
        return this.writeToContract("createConsent", [ownerAddress, baseUri, name], overrides);
    }
    estimateGasToCreateConsent(ownerAddress, baseUri, name) {
        return ResultAsync.fromPromise(this.contract.estimateGas["createConsent"](ownerAddress, baseUri, name), (e) => {
            return this.generateError(e, `Failed to estimate gas with error: ${e}`);
        }).map((estimatedGas) => {
            // TODO: confirm buffer value
            // Increase estimated gas buffer by 10%
            return (estimatedGas * 110n) / 100n;
        });
    }
    // Gets the count of user's deployed Consents
    getUserDeployedConsentsCount(ownerAddress) {
        return ResultAsync.fromPromise(this.contract.getUserDeployedConsentsCount(ownerAddress), (e) => {
            return this.generateError(e, "Unable to call getUserDeployedConsentsCount()");
        }).map((count) => {
            return Number(count);
        });
    }
    // Gets the array of user deployed Consents by index count
    // Index values can be anywhere between the count obtained from getUserDeployedConsentsCount
    // eg. If user has [0x123, 0xabc, 0x456] Consent contracts, query with startingIndex 0 and endingIndex 2 to get full list
    getUserDeployedConsentsByIndex(ownerAddress, startingIndex, endingIndex) {
        return ResultAsync.fromPromise(this.contract.getUserDeployedConsentsByIndex(ownerAddress, startingIndex, endingIndex), (e) => {
            return this.generateError(e, "Unable to call getUserDeployedConsentsByIndex()");
        });
    }
    // get the latest deployed consent address by owner account address
    getUserDeployedConsents(ownerAddress) {
        return this.getUserDeployedConsentsCount(ownerAddress).andThen((count) => {
            return this.getUserDeployedConsentsByIndex(ownerAddress, 0, count);
        });
    }
    // Gets the count of Consent addresses user has specific roles for
    getUserRoleAddressesCount(ownerAddress, role) {
        return ResultAsync.fromPromise(this.contract.getUserConsentAddressesCount(ownerAddress, role), (e) => {
            return this.generateError(e, "Unable to call getUserConsentAddressesCount()");
        }).map((count) => {
            return Number(count);
        });
    }
    // Gets the array of Consent addresses user has specific roles for
    // Index values can be anywhere between the count obtained from getUserRoleAddressesCount
    // eg. If user has [0x123, 0xabc, 0x456] Consent contracts, query with startingIndex 0 and endingIndex 2 to get full list
    getUserRoleAddressesCountByIndex(ownerAddress, role, startingIndex, endingIndex) {
        return ResultAsync.fromPromise(this.contract.getUserRoleAddressesCountByIndex(ownerAddress, role, startingIndex, endingIndex), (e) => {
            return this.generateError(e, "Unable to call getUserRoleAddressesCountByIndex()");
        });
    }
    getDeployedConsents() {
        const eventFilter = this.contract.filters.ConsentDeployed();
        return ResultAsync.fromPromise(this.contract.queryFilter(eventFilter), (e) => {
            return this.generateError(e, "Unable to call contract.queryFilter() for ConsentDeployed event");
        }).map((events) => {
            const consents = [];
            events.forEach((event) => {
                if (event instanceof ethers.EventLog) {
                    if (event.args.consentAddress != null) {
                        consents.push(EVMContractAddress(event.args.consentAddress));
                    }
                }
            });
            return consents;
        });
    }
    // Marketplace functions
    getMaxTagsPerListing() {
        return ResultAsync.fromPromise(this.contract.maxTagsPerListing(), (e) => {
            return this.generateError(e, "Unable to call getMaxTagsPerListing()");
        }).map((num) => {
            return Number(num);
        });
    }
    getListingDuration() {
        return ResultAsync.fromPromise(this.contract.getListingDuration(), (e) => {
            return this.generateError(e, "Unable to call getListingDuration()");
        }).map((num) => {
            return Number(num);
        });
    }
    setListingDuration(listingDuration, overrides) {
        return this.writeToContract("setListingDuration", [listingDuration], overrides);
    }
    setMaxTagsPerListing(maxTagsPerListing, overrides) {
        return this.writeToContract("setMaxTagsPerListing", [maxTagsPerListing], overrides);
    }
    adminRemoveListing(tag, removedSlot, overrides) {
        return this.writeToContract("setMaxTagsPerListing", [tag, removedSlot], overrides);
    }
    getListingDetail(tag, slot) {
        return ResultAsync.fromPromise(this.contract.getListing(tag, slot), (e) => {
            return this.generateError(e, "Unable to call getListing()");
        }).map((listing) => {
            return new MarketplaceListing(BigNumberString(listing.previous.toString()), BigNumberString(listing.next.toString()), listing.consentContract, UnixTimestamp(Number(listing.timeExpiring)), IpfsCID(""), // TODO: Update contract to also return its CID for getListing (only does this with getListingsForward/backward atm)
            slot, tag);
        });
    }
    getListingsForward(tag, startingSlot, numberOfSlots, removeExpired) {
        return ResultAsync.fromPromise(this.contract.getListingsForward(tag, startingSlot, numberOfSlots, removeExpired), (e) => {
            return this.generateError(e, "Unable to call getListingsForward()");
        }).map(([cids, listings]) => {
            return listings
                .map((listing, index) => {
                return new MarketplaceListing(BigNumberString(listing.previous.toString()), BigNumberString(listing.next.toString()), listing.consentContract, UnixTimestamp(Number(listing.timeExpiring)), IpfsCID(cids[index]), listings[index + 1] != null && listing.next != 0n
                    ? BigNumberString(listings[index + 1].previous.toString())
                    : listings[index - 1] != null
                        ? BigNumberString(listings[index - 1].next.toString())
                        : startingSlot, tag);
            })
                .filter((listing) => {
                // Filter out slots with previous and next zeros
                return listing.previous != "0" || listing.next != "0";
            });
        });
    }
    getListingsBackward(tag, startingSlot, numberOfSlots, removeExpired) {
        return ResultAsync.fromPromise(this.contract.getListingsForward(tag, startingSlot, numberOfSlots, removeExpired), (e) => {
            return this.generateError(e, "Unable to call getListingsForward()");
        }).map(([cids, listings]) => {
            return listings
                .map((listing, index) => {
                return new MarketplaceListing(BigNumberString(listing.previous.toString()), BigNumberString(listing.next.toString()), listing.consentContract, UnixTimestamp(Number(listing.timeExpiring)), IpfsCID(cids[index]), listings[index + 1] != null && listing.previous != ethers.MaxUint256
                    ? BigNumberString(listings[index + 1].next.toString())
                    : listings[index - 1] != null
                        ? BigNumberString(listings[index - 1].previous.toString())
                        : startingSlot, tag);
            })
                .filter((listing) => {
                // Filter out slots with previous and next zeros
                return listing.previous != "0" || listing.next != "0";
            });
        });
    }
    getTagTotal(tag) {
        return ResultAsync.fromPromise(this.contract.getTagTotal(tag), (e) => {
            return this.generateError(e, "Unable to call getTagTotal()");
        }).map((count) => {
            return Number(count);
        });
    }
    getListingsByTag(tag, removeExpired) {
        // We get the total number of slots by calling getTagTotal()
        // And if we query the 2^256 - 1 slot by calling getListingDetail(), its previous member variable will point to the highest ranked listing for that tag
        return ResultUtils.combine([
            this.getTagTotal(tag),
            this.getListingDetail(tag, BigNumberString(ethers.MaxUint256.toString())),
        ]).andThen(([tagTotal, listingDetail]) => {
            // The max slot's next points to the highest slot
            const highestRankListingSlot = listingDetail.next;
            // Fetch from the highest to lowest listings
            return this.getListingsForward(tag, highestRankListingSlot, tagTotal, removeExpired);
        });
    }
    getAddressOfConsentCreated(txRes) {
        return txRes.wait().map((receipt) => {
            // Get the hash of the event
            const event = "ConsentDeployed(address,address)";
            const eventHash = ethers.keccak256(ethers.toUtf8Bytes(event));
            // Filter out for the ConsentDeployed event from the receipt's logs
            // returns an array
            const consentDeployedLog = receipt.logs.filter((_log) => _log.topics[0] == eventHash);
            // access the data and topics from the filtered log
            const data = consentDeployedLog[0].data;
            const topics = consentDeployedLog[0].topics;
            // Declare a new interface
            const iface = new ethers.Interface([
                "event ConsentDeployed(address indexed owner, address indexed consentAddress)",
            ]);
            // Decode the log from the given data and topic
            const decodedLog = iface.decodeEventLog("ConsentDeployed", data, topics);
            const deployedConsentAddress = decodedLog.consentAddress;
            return deployedConsentAddress;
        });
    }
    // #region Questionnaires
    getQuestionnaires() {
        return ResultAsync.fromPromise(this.contract.getQuestionnaires(), (e) => {
            return this.generateError(e, "Unable to call getQuestionnaires()");
        });
    }
    addQuestionnaire(ipfsCid, overrides) {
        return this.writeToContract("addQuestionnaire", [ipfsCid], overrides);
    }
    removeQuestionnaire(index, overrides) {
        return this.writeToContract("removeQuestionnaire", [index], overrides);
    }
    // #endregion Questionnaires
    generateContractSpecificError(msg, e, transaction) {
        return new ConsentFactoryContractError(msg, e, transaction);
    }
};
ConsentFactoryContract = __decorate([
    injectable(),
    __metadata("design:paramtypes", [Object, String])
], ConsentFactoryContract);
export { ConsentFactoryContract };
// I listingStruct { at the place where we're using it, and don't have to export here
// Alternative option is to get the deployed Consent addresses through filtering event ConsentDeployed() event
/* // TODO: Replace Promise<any> with correct types returned from ConsentDeployed() and queryFilter()
  public getConsentsDeployedByOwner(
    ownerAddress: EVMAccountAddress,
  ): ResultAsync<EVMContractAddress[], ConsentFactoryContractError> {
    return ResultAsync.fromPromise(
      this.contract.filters.ConsentDeployed(ownerAddress) as Promise<any>,
      (e) => {
        return new ConsentFactoryContractError(
          "Unable to call filters.ConsentDeployed()",
          (e as IBlockchainError).reason,
          e,
        );
      },
    )
      .andThen((_logs) => {
        return ResultAsync.fromPromise(
          this.contract.queryFilter(_logs) as Promise<any>,
          (e) => {
            return new ConsentFactoryContractError(
              "Unable to call filters.ConsentDeployed()",
              (e as IBlockchainError).reason,
              e,
            );
          },
        );
      })
      .map((logs) => {
        return logs.map((log) => log.args.consentAddress);
      });
  }
} */
//# sourceMappingURL=ConsentFactoryContract.js.map