import { EVMAccountAddress, EVMContractAddress, TokenUri, TokenId, CrumbsContractError, HexString, BlockchainCommonErrors } from "@snickerdoodlelabs/objects";
import { ethers } from "ethers";
import { ResultAsync } from "neverthrow";
import { BaseContract } from "../implementations/BaseContract.js";
import { IEthersContractError } from "../implementations/BlockchainErrorMapper";
import { ICrumbsContract } from "../interfaces/ICrumbsContract.js";
import { WrappedTransactionResponse, ContractOverrides } from "../interfaces/objects/index.js";
export declare class CrumbsContract extends BaseContract<CrumbsContractError> implements ICrumbsContract {
    protected providerOrSigner: ethers.Provider | ethers.Signer;
    protected contractAddress: EVMContractAddress;
    constructor(providerOrSigner: ethers.Provider | ethers.Signer, contractAddress: EVMContractAddress);
    getContractAddress(): EVMContractAddress;
    addressToCrumbId(accountAddress: EVMAccountAddress): ResultAsync<TokenId | null, CrumbsContractError | BlockchainCommonErrors>;
    tokenURI(tokenId: TokenId): ResultAsync<TokenUri | null, CrumbsContractError | BlockchainCommonErrors>;
    createCrumb(crumbId: TokenId, tokenUri: TokenUri, overrides?: ContractOverrides): ResultAsync<WrappedTransactionResponse, BlockchainCommonErrors | CrumbsContractError>;
    encodeCreateCrumb(crumbId: TokenId, crumbContent: TokenUri): HexString;
    burnCrumb(crumbId: TokenId, overrides?: ContractOverrides | undefined): ResultAsync<WrappedTransactionResponse, BlockchainCommonErrors | CrumbsContractError>;
    encodeBurnCrumb(crumbId: TokenId): HexString;
    updateTokenURI(crumbId: TokenId, tokenURI: TokenUri, overrides?: ContractOverrides): ResultAsync<WrappedTransactionResponse, BlockchainCommonErrors | CrumbsContractError>;
    protected generateContractSpecificError(msg: string, e: IEthersContractError, transaction: ethers.Transaction | null): CrumbsContractError;
}
//# sourceMappingURL=CrumbsContract.d.ts.map