var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var RewardsContractFactory_1;
import { EVMContractAddress, EVMAccountAddress, RewardsFactoryError, ECreatedRewardType, } from "@snickerdoodlelabs/objects";
import { ethers } from "ethers";
import { injectable } from "inversify";
import { ResultAsync } from "neverthrow";
import { BaseContract } from "../../implementations/BaseContract.js";
import { GasUtils } from "../../implementations/GasUtils.js";
import { ContractsAbis, } from "../../interfaces/objects/index.js";
let RewardsContractFactory = RewardsContractFactory_1 = class RewardsContractFactory extends BaseContract {
    constructor(providerOrSigner, rewardType) {
        super(providerOrSigner, EVMContractAddress(ethers.ZeroAddress), // The rewards contract factory deploys a new contract, hence doesn't have a contract address
        ContractsAbis.ERC721Reward.abi);
        this.providerOrSigner = providerOrSigner;
        this.rewardType = rewardType;
        // Set the correct contract factory based on rewardTypeToDeploy
        this.erc721ContractFactory = new ethers.ContractFactory(ContractsAbis.ERC721Reward.abi, ContractsAbis.ERC721Reward.bytecode, providerOrSigner);
        this.erc20ContractFactory = new ethers.ContractFactory(ContractsAbis.ERC20Reward.abi, ContractsAbis.ERC20Reward.bytecode, providerOrSigner);
        this.oft20RewardContractFactory = new ethers.ContractFactory(ContractsAbis.OFT20Reward.abi, ContractsAbis.OFT20Reward.bytecode, providerOrSigner);
        this.onft721RewardContractFactory = new ethers.ContractFactory(ContractsAbis.ONFT721Reward.abi, ContractsAbis.ONFT721Reward.bytecode, providerOrSigner);
        this.erc1155ContractFactory = new ethers.ContractFactory(ContractsAbis.ERC1155Reward.abi, ContractsAbis.ERC1155Reward.bytecode, providerOrSigner);
        this.rewardTypeToDeploy = rewardType;
    }
    // function to deploy a new ERC721 reward contract
    deployERC721Reward(name, symbol, baseURI, overrides, omitGasFee = false) {
        return GasUtils.getGasFee(this.providerOrSigner).andThen((gasFee) => {
            let contractOverrides = {
                ...gasFee,
                ...overrides,
            };
            // If the chain does not support EIP-1559, remove the gas fee override and only maintain the override passed in from the chain service
            if (omitGasFee == true) {
                contractOverrides = {
                    ...overrides,
                };
            }
            return this.writeToContractFactory("deploy", [name, symbol, baseURI], ECreatedRewardType.ERC721, contractOverrides, true);
        });
    }
    estimateGasToDeployERC721Contract(name, symbol, baseURI) {
        return ResultAsync.fromPromise(this.erc721ContractFactory.getDeployTransaction(name, symbol, baseURI), (e) => {
            return this.generateError(e, "Unable to get deploy transaction for contract deployment for ERC721 contract");
        })
            .andThen((deployTransaction) => {
            return ResultAsync.fromPromise(this.providerOrSigner.estimateGas(deployTransaction), (e) => {
                return this.generateError(e, "Attempting to estimate gas for contract deployment");
            });
        })
            .map((estimatedGas) => {
            // Increase estimated gas buffer by 20%
            return (estimatedGas * 120n) / 100n;
        });
    }
    // function to deploy a new ERC20 reward contract
    deployERC20Reward(name, symbol, overrides, omitGasFee = false) {
        return GasUtils.getGasFee(this.providerOrSigner).andThen((gasFee) => {
            let contractOverrides = {
                ...gasFee,
                ...overrides,
            };
            // If the chain does not support EIP-1559, remove the gas fee override and only maintain the override passed in from the chain service
            if (omitGasFee == true) {
                contractOverrides = {
                    ...overrides,
                };
            }
            return this.writeToContractFactory("deploy", [name, symbol], ECreatedRewardType.ERC20, contractOverrides, true);
        });
    }
    estimateGasToDeployERC20Contract(name, symbol) {
        return ResultAsync.fromPromise(this.erc20ContractFactory.getDeployTransaction(name, symbol), (e) => {
            return this.generateError(e, "Unable to get deploy transaction for contract deployment for ERC20 contract");
        })
            .andThen((deployTransaction) => {
            return ResultAsync.fromPromise(this.providerOrSigner.estimateGas(deployTransaction), (e) => {
                return this.generateError(e, "Attempting to estimate gas for contract deployment");
            });
        })
            .map((estimatedGas) => {
            // Increase estimated gas buffer by 20%
            return (estimatedGas * 120n) / 100n;
        });
    }
    // function to deploy a new ERC1155 reward contract
    deployERC1155Reward(numberOfRewards, tokenURIs, overrides, omitGasFee = false) {
        return GasUtils.getGasFee(this.providerOrSigner).andThen((gasFee) => {
            let contractOverrides = {
                ...gasFee,
                ...overrides,
            };
            // If the chain does not support EIP-1559, remove the gas fee override and only maintain the override passed in from the chain service
            if (omitGasFee == true) {
                contractOverrides = {
                    ...overrides,
                };
            }
            return this.writeToContractFactory("deploy", [numberOfRewards, tokenURIs], ECreatedRewardType.ERC1155, contractOverrides, true);
        });
    }
    estimateGasToDeployERC1155Contract(numberOfRewards, tokenURIs) {
        return ResultAsync.fromPromise(this.erc1155ContractFactory.getDeployTransaction(numberOfRewards, tokenURIs), (e) => {
            return this.generateError(e, "Unable to get deploy transaction for contract deployment for OFT20 Reward contract");
        })
            .andThen((deployTransaction) => {
            return ResultAsync.fromPromise(this.providerOrSigner.estimateGas(deployTransaction), (e) => {
                return this.generateError(e, "Attempting to estimate gas for contract deployment");
            });
        })
            .map((estimatedGas) => {
            // Increase estimated gas buffer by 20%
            return (estimatedGas * 120n) / 100n;
        });
    }
    // function to deploy a new OFT20Reward reward contract
    deployOFT20Reward(name, symbol, layerZeroEndpoint, overrides, omitGasFee = false) {
        return GasUtils.getGasFee(this.providerOrSigner).andThen((gasFee) => {
            let contractOverrides = {
                ...gasFee,
                ...overrides,
            };
            // If the chain does not support EIP-1559, remove the gas fee override and only maintain the override passed in from the chain service
            if (omitGasFee == true) {
                contractOverrides = {
                    ...overrides,
                };
            }
            return this.writeToContractFactory("deploy", [name, symbol, layerZeroEndpoint], ECreatedRewardType.OFT20, contractOverrides, true);
        });
    }
    estimateGasToDeployOFT20RewardContract(name, symbol, layerZeroEndpoint) {
        return ResultAsync.fromPromise(this.oft20RewardContractFactory.getDeployTransaction(name, symbol, layerZeroEndpoint), (e) => {
            return this.generateError(e, "Unable to get deploy transaction for contract deployment for OFT20 Reward contract");
        })
            .andThen((deployTransaction) => {
            return ResultAsync.fromPromise(this.providerOrSigner.estimateGas(deployTransaction), (e) => {
                return this.generateError(e, "Attempting to estimate gas for contract deployment");
            });
        })
            .map((estimatedGas) => {
            // Increase estimated gas buffer by 20%
            return (estimatedGas * 120n) / 100n;
        });
    }
    // function to deploy a new OFT20Reward reward contract
    deployONFT721Reward(name, symbol, baseURI, minGasToTransfer, layerZeroEndpoint, overrides, omitGasFee = false) {
        return GasUtils.getGasFee(this.providerOrSigner).andThen((gasFee) => {
            let contractOverrides = {
                ...gasFee,
                ...overrides,
            };
            // If the chain does not support EIP-1559, remove the gas fee override and only maintain the override passed in from the chain service
            if (omitGasFee == true) {
                contractOverrides = {
                    ...overrides,
                };
            }
            return this.writeToContractFactory("deploy", [name, symbol, baseURI, minGasToTransfer, layerZeroEndpoint], ECreatedRewardType.ONFT721, contractOverrides, true);
        });
    }
    estimateGasToDeployONFT721RewardContract(name, symbol, baseURI, minGasToTransfer, layerZeroEndpoint) {
        return ResultAsync.fromPromise(this.onft721RewardContractFactory.getDeployTransaction(name, symbol, baseURI, minGasToTransfer, layerZeroEndpoint), (e) => {
            return this.generateError(e, "Unable to get deploy transaction for contract deployment for ONFT721 Reward contract");
        })
            .andThen((deployTransaction) => {
            return ResultAsync.fromPromise(this.providerOrSigner.estimateGas(deployTransaction), (e) => {
                return this.generateError(e, "Attempting to estimate gas for contract deployment");
            });
        })
            .map((estimatedGas) => {
            // Increase estimated gas buffer by 20%
            return (estimatedGas * 120n) / 100n;
        });
    }
    generateContractSpecificError(msg, e, transaction) {
        return new RewardsFactoryError(msg, e, transaction);
    }
    // Takes the factory's deploy function name and params, submits the transaction and returns a WrappedTransactionResponse
    writeToContractFactory(functionName, functionParams, rewardType, overrides, isDeployingContract = false) {
        let contractFactory = this.erc721ContractFactory;
        let abi;
        if (rewardType == ECreatedRewardType.ERC20) {
            contractFactory = this.erc20ContractFactory;
            abi = ContractsAbis.ERC20Reward.abi;
        }
        if (rewardType == ECreatedRewardType.ERC1155) {
            contractFactory = this.erc1155ContractFactory;
            abi = ContractsAbis.ERC1155Reward.abi;
        }
        if (rewardType == ECreatedRewardType.OFT20) {
            contractFactory = this.erc1155ContractFactory;
            abi = ContractsAbis.OFT20Reward.abi;
        }
        if (rewardType == ECreatedRewardType.ONFT721) {
            contractFactory = this.erc1155ContractFactory;
            abi = ContractsAbis.ONFT721Reward.abi;
        }
        return ResultAsync.fromPromise(contractFactory[functionName](...functionParams, {
            ...overrides,
        }), (e) => {
            return this.generateError(e, `Unable to call ${functionName}()`);
        }).map((transactionResponse) => {
            // If we are deploying a contract, the deploy() call returns an ethers.Contract object and the txresponse is under the deployTransaction property
            return RewardsContractFactory_1.buildWrappedTransactionResponse(isDeployingContract == true
                ? transactionResponse.deploymentTransaction()
                : transactionResponse, EVMContractAddress(""), EVMAccountAddress(this.providerOrSigner?.address), functionName, functionParams, abi);
        });
    }
};
RewardsContractFactory = RewardsContractFactory_1 = __decorate([
    injectable(),
    __metadata("design:paramtypes", [Object, String])
], RewardsContractFactory);
export { RewardsContractFactory };
//# sourceMappingURL=RewardsContractFactory.js.map